package rwi.eclipse.team.ccvs.ssl;

/*******************************************************************************
 * Copyright (c) 2003 Rolf Wilms (rolf_wilms@hotmail.com).
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 * 
 *******************************************************************************/

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class SSLPlugin extends AbstractUIPlugin {
	//The shared instance.
	private static SSLPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;

	// Keys for preferences
	public static String KEY_CLIENT_KEYS = "CLIENT_KEYS";
	public static String KEY_SERVER_CERTIFICATES = "SERVER_CERTIFICATES";
	public static String KEY_KEYSTORE_FILE = "KEYSTORE_FILE";
	public static String KEY_KEYSTORE_PASSWORD = "KEYSTORE_PASSWORD";
	public static String KEY_TRUSTSTORE_FILE = "TRUSTSTORE_FILE";
	public static String KEY_TRUSTSTORE_PASSWORD = "TRUSTSTORE_PASSWORD";

	// Values for preferences
	public static String VAL_SERVER_CERTIFICATES_DEFAULT_TRUSTSTORE = "DEFAULT_TRUSTSTORE";
	public static String VAL_SERVER_CERTIFICATES_ACCEPT_ALL = "ACCEPT_ALL";
	public static String VAL_SERVER_CERTIFICATES_CUSTOM_TRUSTSTORE = "CUSTOM_TRUSTSTORE";

	public static String VAL_CLIENT_KEYS_DEFAULT_KEYSTORE = "DEFAULT_KEYSTORE";
	public static String VAL_CLIENT_KEYS_CUSTOM_KEYSTORE = "CUSTOM_KEYSTORE";

	// The socket factory for new SSL sockets
	private SSLSocketFactory sslSocketFactory = null;

	// Our Plugin ID
	public static String PLUGIN_ID = "rwi.eclipse.team.ccvs.ssl";
	
	/**
	 * The constructor.
	 */
	public SSLPlugin() {
		super();
		plugin = this;
	}

	public void start(BundleContext context) throws Exception {
		super.start(context);
		try {
			resourceBundle =
				ResourceBundle.getBundle(
					"rwi.eclipse.team.ccvs.ssl.SSLPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
		
		// Initialize preference defaults
		getPreferenceStore().setDefault(KEY_SERVER_CERTIFICATES, VAL_SERVER_CERTIFICATES_ACCEPT_ALL);
		getPreferenceStore().setDefault(KEY_CLIENT_KEYS, VAL_CLIENT_KEYS_DEFAULT_KEYSTORE);

		// Invalidate sslSocketFactory each time the preferences are changed.
		// Will laziely create new ssSocketFactory with new preferences. 
		getPreferenceStore()
			.addPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				sslSocketFactory = null;
			}
		});
	}

	/**
	 * Returns the shared instance.
	 */
	public static SSLPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = SSLPlugin.getDefault().getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	// Return the SSL socket factory for new SSLSockets
	public SSLSocketFactory getSSLSocketFactory() throws IOException {
		if (sslSocketFactory == null) {
			try {
				// lazy creation
				createSSLSocketFactory();
			} catch (KeyManagementException e) {
				throw new IOException(e.getMessage());
			} catch (CertificateException e) {
				throw new IOException(e.getMessage());
			} catch (KeyStoreException e) {
				throw new IOException(e.getMessage());
			} catch (NoSuchAlgorithmException e) {
				throw new IOException(e.getMessage());
			} catch (UnrecoverableKeyException e) {
				throw new IOException(e.getMessage());
			}
		}
		return sslSocketFactory;
	}

	/**
	 * 
	 */
	private void createSSLSocketFactory()
		throws
			CertificateException,
			IOException,
			KeyStoreException,
			NoSuchAlgorithmException,
			UnrecoverableKeyException,
			KeyManagementException {
				
		KeyStore ks = null;
		// Create custom keystore if chosen
		if (getUseCustomKeystore()) {
			ks = createKeyStore(KEY_KEYSTORE_FILE, KEY_KEYSTORE_PASSWORD);
		}

		KeyStore ts = null;
		// Create custom truststore if chosen
		if (getUseCustomTruststore()) {
			ts = createKeyStore(KEY_TRUSTSTORE_FILE, KEY_TRUSTSTORE_PASSWORD);
		}

		KeyManager[] km = null;
		TrustManager[] tm = null;

		if (ks != null) {
			// get the KeyManagers for the keystore
			KeyManagerFactory kmf =
				KeyManagerFactory.getInstance(
					KeyManagerFactory.getDefaultAlgorithm());
			kmf.init(
				ks,
				getPreferenceStore()
					.getString(KEY_KEYSTORE_PASSWORD)
					.toCharArray());
			km = kmf.getKeyManagers();
		} // Otherwise km is null (== default)

		if (ts != null) {
			// get the TrustManagers for the truststore
			TrustManagerFactory tmf =
				TrustManagerFactory.getInstance(
					TrustManagerFactory.getDefaultAlgorithm());
			tmf.init(ts);
			tm = tmf.getTrustManagers();
		} else if (getAcceptAllCertificates()) {
			// use a TrustManager accepting all certificates
			tm = new TrustManager[] { new AllAcceptingX509TrustManager()};
		} // Otherwise tm is null (== default)

		// TODO helpme: using "SSL" works but what are the alternatives and what should really be used??
		SSLContext sslContext = SSLContext.getInstance("SSL");

		// Initialize the SSLContext according to our preferences.		
		// A null km or tm means default
		sslContext.init(km, tm, null);
		
		// now we can create the SSLSocketFactory
		sslSocketFactory = sslContext.getSocketFactory();
	}

	private KeyStore createKeyStore(String keyFilename, String keyPassword)
		throws
			KeyStoreException,
			NoSuchAlgorithmException,
			CertificateException,
			IOException {
		KeyStore ks;
		String keystoreFilename = getPreferenceStore().getString(keyFilename);
		String keystorePwd = getPreferenceStore().getString(keyPassword);
		InputStream ksFile = new FileInputStream(keystoreFilename);
		try {
			ks = KeyStore.getInstance(KeyStore.getDefaultType());
			ks.load(ksFile, keystorePwd.toCharArray());
		} finally {
			if (ksFile != null) {
				ksFile.close();
			}
		}
		return ks;
	}

	public boolean getUseCustomKeystore() {
		return getPreferenceStore().getString(KEY_CLIENT_KEYS).equals(
		VAL_CLIENT_KEYS_CUSTOM_KEYSTORE);
	}

	public boolean getUseCustomTruststore() {
		return getPreferenceStore().getString(KEY_SERVER_CERTIFICATES).equals(
			VAL_SERVER_CERTIFICATES_CUSTOM_TRUSTSTORE);
	}

	public boolean getAcceptAllCertificates() {
		return getPreferenceStore().getString(KEY_SERVER_CERTIFICATES).equals(
			VAL_SERVER_CERTIFICATES_ACCEPT_ALL);
	}
}
